package org.commoncrawl.mapreduce.gethtml;

import java.io.IOException;

import org.apache.commons.io.IOUtils;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.log4j.Logger;
import org.archive.io.ArchiveReader;
import org.archive.io.ArchiveRecord;

public class HtmlMap {
	private static final Logger LOG = Logger.getLogger(HtmlMap.class);
	protected static enum MAPPERCOUNTER {
		RECORDS_IN,
		EXCEPTIONS
	}

	protected static class HtmlMapper extends Mapper<Text, ArchiveReader, Text, NullWritable> {
		private Text outKey = new Text();
		// The HTML regular expression is case insensitive (?i), avoids closing tags (?!/),
		// tries to find just the tag name before any spaces, and then consumes any other attributes.

		@Override
		public void map(Text key, ArchiveReader value, Context context) throws IOException {
			// Compile the regular expression once as it will be used continuously
			
			for (ArchiveRecord r : value) {
				try {
					LOG.debug(r.getHeader().getUrl() + " -- " + r.available());
					// We're only interested in processing the responses, not requests or metadata
					if (r.getHeader().getMimetype().equals("application/http; msgtype=response")) {
						// Convenience function that reads the full message into a raw byte array
						byte[] rawData = IOUtils.toByteArray(r, r.available());
						String content = new String(rawData);
						// The HTTP header gives us valuable information about what was received during the request
						String headerText = content.substring(0, content.indexOf("\r\n\r\n"));
						
						// In our task, we're only interested in text/html, so we can be a little lax
						// TODO: Proper HTTP header parsing + don't trust headers
						if (headerText.contains("Content-Type: text/html") ||
							headerText.contains("Content-Type:text/html")) {
							
							
							// Only extract the body of the HTTP response when necessary
							// Due to the way strings work in Java, we don't use any more memory than before
							String body = content.substring(content.indexOf("\r\n\r\n") + 4);
							
							// free memory
							content = null;
							
							body = body + " commoncrawl hadoop output";
							
							String lowerContent = body.toLowerCase();

							// search keywords in HTML
							if (lowerContent.contains("gambler")
									|| lowerContent.contains("rich")
									|| lowerContent.contains("name list")
									|| lowerContent.contains("2nd rich generation")
									|| lowerContent
											.contains("2nd official generation")
									|| lowerContent.contains("gambler addict")
									|| lowerContent.contains("gamble")
									|| lowerContent.contains("shanxi")
									|| lowerContent.contains("macau")
									|| lowerContent.contains("rich businessman")
									|| lowerContent.contains("tour")
									|| lowerContent.contains("smoking")) {
								
								outKey.set(body);
								context.write(outKey, NullWritable.get());
								
							}
							body = null;
							lowerContent = null;
						}
					}
				}
				catch (Exception ex) {
					LOG.error("Caught Exception", ex);
					context.getCounter(MAPPERCOUNTER.EXCEPTIONS).increment(1);
				}
			}
		}
	}
}
